const { GObject, St, Clutter, GLib, Gio } = imports.gi;

const ExtensionUtils = imports.misc.extensionUtils;
const Me = ExtensionUtils.getCurrentExtension();

const Util = imports.misc.util;
const Main = imports.ui.main;
const PanelMenu = imports.ui.panelMenu;
const PopupMenu = imports.ui.popupMenu;

const Gettext = imports.gettext;
const Domain = Gettext.domain(Me.metadata.uuid);
const _ = Domain.gettext;

const Status = {
    ONLINE: 1,
    OFFLINE: 2,
    ERROR: 3,
    UNDEFINED: 4,
};

const Icon = {
    ONLINE: 'online.svg',
    OFFLINE: 'offline.svg',
    ERROR: 'error.svg',
    UNDEFINED: 'undefined.svg',
};

const Settings = {
    SERVER_MAC: 'servermac',
    SERVER_IP: 'serverip',
    SERVER_POLL: 'serverpoll',
};

const ServerIndicator = GObject.registerClass(
    class Indicator extends PanelMenu.Button {
        _init() {
            super._init(0.0, _('Server'));
            this._indicatorStatus = {
                lastResponse: null,
                status: Status.UNDEFINED,
            };
            this._settings = ExtensionUtils.getSettings('org.gnome.shell.extensions.Server');
            this._debugMode = this._settings.get_boolean('serverdebug');
            this._setupWidgets();

            this._mainLoop();
            this._startMainLoop();
        }

        _setupWidgets() {
            this._menuLayout = new St.BoxLayout();
            this._ServerIcon = new St.Icon({ style_class: 'system-status-icon' });
            this._setIndicatorIcon(Icon.UNDEFINED);
            this._menuLayout.add_actor(this._ServerIcon);
            this.add_actor(this._menuLayout);
            this._setupPopupMenu();
        }

        _setupPopupMenu() {
            this._settingsMenuItem = new PopupMenu.PopupMenuItem(_('Settings'));
            this._settingsMenuItem.connect('activate', () => {
                Util.spawn(['gnome-extensions', 'prefs', Me.metadata.uuid]);
            });
            this.menu.addMenuItem(this._settingsMenuItem);
        }

        _setIndicatorIcon(iconFileName) {
            const iconPath = Me.dir
                .get_child('icons')
                .get_child(iconFileName)
                .get_path();
            this._ServerIcon.set_gicon(Gio.Icon.new_for_string(iconPath));
        }

        _setIndicator(status) {
            if (status == Status.ONLINE) {
                if ( this._indicatorStatus.status !== Status.ONLINE ) {
                    this._setIndicatorIcon(Icon.ONLINE);
                    this._log('Set status ONLINE');
                    this._indicatorStatus.status = Status.ONLINE;
                    if (this._wakeonlanMenuItem) {
                        this._log('Remove WOL menu item');
                        this._wakeonlanMenuItem.disconnect();
                        this._wakeonlanMenuItem.destroy();
                    }    
                }
                return;
            } 
            
            if (status == Status.OFFLINE) {
                if ( this._indicatorStatus.status !== Status.OFFLINE ) {
                    this._setIndicatorIcon(Icon.OFFLINE);
                    this._log('Set status OFFLINE');
                    this._indicatorStatus.status = Status.OFFLINE;
                    if (!this._wakeonlanMenuItem) {
                        this._log('Add WOL menu item');
                        this._wakeonlanMenuItem = new PopupMenu.PopupMenuItem(_('Start Server'));
                        this._wakeonlanMenuItem.connect('activate', () => {
                            this._wakeup();
                        });
                        this.menu.addMenuItem(this._wakeonlanMenuItem);
                    }
                }
                return;
            }

            if (status == Status.ERROR) {
                if ( this._indicatorStatus.status !== Status.ERROR ) {
                    this._setIndicatorIcon(Icon.ERROR);
                    this._log('Set status ERROR');
                    this._indicatorStatus.status = Status.ERROR;
                    if (this._wakeonlanMenuItem) {
                        this._log('Remove WOL menu item');
                        this._wakeonlanMenuItem.disconnect();
                        this._wakeonlanMenuItem.destroy();
                    }    
                }
                return;
            }

            if (status == Status.UNDEFINED) {
                if ( this._indicatorStatus.status !== Status.UNDEFINED ) {
                    this._setIndicatorIcon(Icon.UNDEFINED);
                    this._log('Set status UNDEFINED');
                    this._indicatorStatus.status = Status.UNDEFINED;
                    if (this._wakeonlanMenuItem) {
                        this._log('Remove WOL menu item');
                        this._wakeonlanMenuItem.disconnect();
                        this._wakeonlanMenuItem.destroy();
                    }    
                }
                return;
            }
        }

        _wakeup() {
            try {
                let proc = Gio.Subprocess.new(
                    ['wakeonlan', this._settings.get_string('servermac')], 
                    Gio.SubprocessFlags.STDOUT_PIPE | Gio.SubprocessFlags.STDERR_PIPE);
                proc.communicate_utf8_async(null, null, (proc, res) => {
                    try {
                        Main.notify(Me.metadata.name, _('Start Server now. Please wait...'))
                        let [, stdout, stderr] = proc.communicate_utf8_finish(res);
                    } catch (e) {
                        Main.notify(Me.metadata.name, _('Start Server failed'))
                        this._log('Executing Wake On LAN failed. ' + e);
                    }
                });
            } catch (e) {
                Main.notify(Me.metadata.name, _('Start Server failed. Already installed wakeonlan?'))
                this._log('Wake On LAN failed. Already installed wakeonlan? ' + e);
            }
        }

        _startMainLoop() {
            this._mainLoopTimeout = GLib.timeout_add_seconds(
                GLib.PRIORITY_DEFAULT,
                this._settings.get_int(Settings.SERVER_POLL),
                () => {
                    this._mainLoop();
                    return true;
                }
            );
        }

        _stopMainLoop() {
            if (this._mainLoopTimeout) {
                GLib.source_remove(this._mainLoopTimeout);
                this._mainLoopTimeout = null;
            }
            this._connection = null;
        }

        async _mainLoop() {
            try {
                let proc = Gio.Subprocess.new(
                    ['ping', '-q', '-c1', '-w5', this._settings.get_string('serverip')], 
                    Gio.SubprocessFlags.STDOUT_PIPE | Gio.SubprocessFlags.STDERR_PIPE);
                proc.communicate_utf8_async(null, null, (proc, res) => {
                    try {
                        let [, stdout, stderr] = proc.communicate_utf8_finish(res);
        
                        if (!proc.get_successful()) {
                            this._log('Ping failed');
                            this._setIndicator(Status.OFFLINE)
                        } else {
                            this._log('Ping success');
                            this._setIndicator(Status.ONLINE)
                        }
                    } catch (e) {
                        this._setIndicator(Status.UNDEFINED)
                        this._log('Ping failed. ' + e);
                    }
                });
            } catch (e) {
                this._setIndicator(Status.ERROR)
                this._log('Ping failed. ' + e);
            }
        }

        _log(msg) {
            if (this._debugMode)
                log(`${Me.metadata.name}: ${msg}`);
        }

        _onDestroy() {
            this._log('Indicator._onDestroy()');
            this._stopMainLoop();
            this._menuLayout = null;
            this._indicatorStatus = null;
            this._settings = null;
            super._onDestroy();
        }
    }
);

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// CLASS EXTENSION ////////////////////////////////////////////////////////////////////////////////////////////////////
class Extension {
    constructor(uuid) {
        this._uuid = uuid;
    }
    
    enable() {
        this._serverIndicator = new ServerIndicator();
        Main.panel.addToStatusArea(this._uuid, this._serverIndicator);
    }
    
    disable() {
        this._serverIndicator.destroy();
        this._serverIndicator = null;
    }
}


function init(meta) {
    ExtensionUtils.initTranslations(Me.metadata.uuid);
    return new Extension(meta.uuid);
}